from infi.pyutils.contexts import contextmanager
import ctypes

### Functions and definitions for installation
setupapi = ctypes.windll.setupapi
newdev = ctypes.windll.newdev
GetLastError = ctypes.windll.kernel32.GetLastError

# definitions from windows.h
INVALID_HANDLE_VALUE = -1
MAX_PATH = 260
ERROR_NO_MORE_ITEMS = 259

# definitions from setupapi.h
SPDRP_HARDWAREID = 0x00000001
DI_ENUMSINGLEINF = 0x00010000
DI_FLAGSEX_ALLOWEXCLUDEDDRVS = 0x00000800
SPDIT_CLASSDRIVER = 0x00000001
DIGCF_ALLCLASSES = 0x00000004
LINE_LEN = 256
SOFTWARE_REPOSITORY = 'http://bootsrv.infinidat.com/clientapps'

class SP_DEVINFO_DATA(ctypes.Structure):
    _fields_ = [("cbSize", ctypes.c_ulong),
                ("classGuid", ctypes.c_char * 16),
                ("DevInst", ctypes.c_ulong),
                ("Reserved", ctypes.c_void_p)]
    def __init__(self):
        self.cbSize = ctypes.sizeof(self)

class SP_DRVINFO_DATA_W(ctypes.Structure):
    _fields_ = [("cbSize", ctypes.c_ulong),
                ("DriverType", ctypes.c_ulong),
                ("Reserved", ctypes.c_void_p),
                # we don't use c_wchar and c_ulonglong (but c_char*2 and c_ulong*2) because of padding issues
                ("Description", ctypes.c_char * 2 * LINE_LEN),
                ("MfgName", ctypes.c_char * 2 * LINE_LEN),
                ("ProviderName", ctypes.c_char * 2 * LINE_LEN),
                ("FileTime", ctypes.c_ulong * 2),
                ("DriverVersion", ctypes.c_ulong * 2)]
    def __init__(self):
        self.cbSize = ctypes.sizeof(self)

class SP_DEVINSTALL_PARAMS(ctypes.Structure):
    _fields_ = [("cbSize", ctypes.c_ulong),
                ("Flags", ctypes.c_ulong),
                ("FlagsEx", ctypes.c_ulong),
                ("hwndParent", ctypes.c_void_p),
                ("InstallMsgHandler", ctypes.c_void_p),
                ("InstallMsgHandlerContext", ctypes.c_void_p),
                ("FileQueue", ctypes.c_void_p),
                ("ClassInstallReserved", ctypes.c_void_p),
                ("Reserved", ctypes.c_ulong),
                ("DriverPath", ctypes.c_char * MAX_PATH)]
    def __init__(self):
        self.cbSize = ctypes.sizeof(self)

def _copy_inf(path):
    # see documentation of _install_inf, this implements step 1
    copied_inf_path = ctypes.c_buffer(MAX_PATH)
    res = setupapi.SetupCopyOEMInfA(path, "", 0, 0, ctypes.byref(copied_inf_path), len(copied_inf_path), None, None)
    assert res, "SetupCopyOEMInfA failed with {}".format(GetLastError())
    return copied_inf_path.value

def _set_fc_device(hdevinfo, devinfo):
    # see documentation of _install_inf, this implements step 2
    EMULEX_HARDWARE_ID = "VEN_10DF&DEV_F100"        # see comment in __init__.py:_has_emulex_device
    dev_hwid = ctypes.c_buffer(1024 * 40)           # buffer to hold each device's hardware ids for comparison
    i = 0
    # enum PCI devices, find the emulex device according to the hardware ids
    while True:
        res = setupapi.SetupDiEnumDeviceInfo(hdevinfo, i, ctypes.byref(devinfo))
        if not res and GetLastError() == ERROR_NO_MORE_ITEMS:
            raise Exception("Couldn't find Emulex device in PCI device enumeration")
        assert res, "SetupDiEnumDeviceInfo failed with {}".format(GetLastError())
        res = setupapi.SetupDiGetDeviceRegistryPropertyA(hdevinfo, ctypes.byref(devinfo),
                                                         SPDRP_HARDWAREID, None,
                                                         ctypes.byref(dev_hwid), len(dev_hwid), None)
        assert res, "SetupDiGetDeviceRegistryPropertyA failed with {}".format(GetLastError())
        if EMULEX_HARDWARE_ID in dev_hwid.raw:
            break
        i += 1
    res = setupapi.SetupDiSetSelectedDevice(hdevinfo, ctypes.byref(devinfo))
    assert res, "SetupDiSetSelectedDevice failed with {}".format(GetLastError())

def _set_inf_in_device_install_params(hdevinfo, devinfo, inf_path):
    # see documentation of _install_inf, this implements step 3
    # Reference: http://support.microsoft.com/kb/889763
    dev_install_params = SP_DEVINSTALL_PARAMS()
    res = setupapi.SetupDiGetDeviceInstallParamsA(hdevinfo, ctypes.byref(devinfo), ctypes.byref(dev_install_params))
    assert res, "SetupDiGetDeviceInstallParamsA failed with {}".format(GetLastError())
    dev_install_params.DriverPath = inf_path
    dev_install_params.Flags |= DI_ENUMSINGLEINF
    dev_install_params.FlagsEx |= DI_FLAGSEX_ALLOWEXCLUDEDDRVS
    res = setupapi.SetupDiSetDeviceInstallParamsA(hdevinfo, ctypes.byref(devinfo), ctypes.byref(dev_install_params))
    assert res, "SetupDiSetDeviceInstallParamsA failed with {}".format(GetLastError())
    res = setupapi.SetupDiBuildDriverInfoList(hdevinfo, ctypes.byref(devinfo), SPDIT_CLASSDRIVER)
    assert res, "SetupDiBuildDriverInfoList failed with {}".format(GetLastError())

def _set_fc_driver(hdevinfo, devinfo):
    # see documentation of _install_inf, this implements step 4
    drvinfo = SP_DRVINFO_DATA_W()
    res = setupapi.SetupDiEnumDriverInfoW(hdevinfo, ctypes.byref(devinfo), SPDIT_CLASSDRIVER, 0, ctypes.byref(drvinfo))
    assert res, "SetupDiEnumDriverInfoW failed with {}".format(GetLastError())
    res = setupapi.SetupDiSetSelectedDriverW(hdevinfo, ctypes.byref(devinfo), ctypes.byref(drvinfo))
    assert res, "SetupDiSetSelectedDriverW failed with {}".format(GetLastError())

def _install_inf(path):
    """ We can't use the high level functions to install the driver:
    - DiInstallDriver doesn't work because it doesn't let us specify the device to update and the driver to install.
    - UpdateDriverForPlugAndPlayDevices doesn't work because it doesn't let us specify the driver to install.
    We must specify both device and driver explicitly because Windows doesn't recognize their compatibility.
    So we use the lower-level function InstallSelectedDriver which lets us select both, but for that we need a lot
    of preparation. We need to:
    1. copy the inf to the inf store
    2. enumerate the devices to find the Emulex\Fibre Channel (sometimes Windows recognizes it's Emulex, sometimes not)
       and set that device in a devinfo handle
    3. Setup the DeviceInstallParams to search for the drivers in the inf file
    4. Enumerate the drivers in the inf file (we will always take the first driver, not really enuemrate)
       and set that driver in the devinfo handle
    5. Call the installation function.
    See remarks section in MSDN documentation of InstallSelectedDriver
    """
    copied_inf_path = _copy_inf(path)
    hdevinfo = setupapi.SetupDiGetClassDevsA(None, "PCI", None, DIGCF_ALLCLASSES)
    assert hdevinfo != INVALID_HANDLE_VALUE
    try:
        devinfo = SP_DEVINFO_DATA()
        _set_fc_device(hdevinfo, devinfo)
        _set_inf_in_device_install_params(hdevinfo, devinfo, copied_inf_path)
        _set_fc_driver(hdevinfo, devinfo)
        res = newdev.InstallSelectedDriver(None, hdevinfo, 0, False, False)
        assert res, "InstallSelectedDriver failed with {}".format(GetLastError())
    finally:
        setupapi.SetupDiDestroyDeviceInfoList(hdevinfo)

### Functions to download the appropriate driver

def _get_winver():
    import re
    from platform import version
    major, minor = re.match("(\d+)\.(\d+)", version()).groups()
    major, minor = int(major), int(minor)
    version_to_name = {
        (6, 0): 'win2008',
        (6, 1): 'win2008R2',
        (6, 2): 'win2012',
        (6, 3): 'win2012R2',
    }
    return version_to_name[(major, minor)]

def _get_arch():
    from platform import architecture
    return {"32bit": "x86", "64bit": "x64"}[architecture()[0]]

@contextmanager
def _download_driver():
    import os
    import tarfile
    from tempfile import mkdtemp
    from shutil import rmtree
    from urllib import urlretrieve
    temp_dir = mkdtemp()
    try:
        driver_tar_name = "FC-10.0.720.0-51.tar.gz"
        tar_url = SOFTWARE_REPOSITORY + "/drivers/Emulex/" + driver_tar_name
        tar_path = os.path.join(temp_dir, driver_tar_name)
        urlretrieve(tar_url, tar_path)
        tarfile.open(tar_path).extractall(temp_dir)
        arch = _get_arch()
        winver = _get_winver()
        driver_dir = os.path.join(temp_dir, "FC-10.0.720.0-51", arch, winver)
        yield driver_dir
    finally:
        rmtree(temp_dir)

### Main

def install():
    import os
    with _download_driver() as driver_dir:
        _install_inf(os.path.join(driver_dir, "oemsetup.inf"))

if __name__ == "__main__":
    # this file is run remotely as a standalone script - to continue the installation locally on the remote host
    install()
