from infi.vendata.jslave.fabric.utils import print_result
from fabric.api import cd, put, run, abort, settings, task
from ...download import download_file_from_repository_on_remote_host
from ..utils import user_context, reconnect, untar, cmmi

GIT_VERSION = "1.9.4"
GIT_REPR = "git-{}".format(GIT_VERSION)
GIT_ARCHIVE_NAME = ".".join([GIT_REPR, "tar", "gz"])
MIN_GIT_VERSION = "1.8.4.1"  # Jenkins git-plugin 2.0.4 needs `git fetch --progress`

PORTABLE_GIT_REPR = "PortableGit-1.9.0-preview20140217"
PORTABLE_GIT_TAR = ".".join([PORTABLE_GIT_REPR, "tar"])
PORTABLE_GIT_ARCHIVE_NAME = ".".join([PORTABLE_GIT_TAR, "gz"])

@task
def install_and_exit_if_needed():
    from .. import runtime
    if not verify():
        if not runtime.verify():
            runtime.install()
        install()
        raise SystemExit(1)

@task
@print_result
def verify():
    from ..utils import determine_platform
    from pkg_resources import parse_version
    with user_context(), settings(warn_only=True):
        version_string = run("git --version").stdout
    expected_prefix = "git version "
    if not version_string.startswith(expected_prefix):
        return False
    version_string = version_string[len(expected_prefix):]
    return parse_version(version_string) >= parse_version(MIN_GIT_VERSION)

def _install_from_source():
    cmmi("git/" + GIT_ARCHIVE_NAME, configure_options="--prefix=/usr --without-tcltk")

def _install_from_apt():
    from ..utils.linux import apt_get, is_ppa_capable, add_ppa_repository
    from fabric.contrib.files import exists
    with settings(warn_only=True):
        if is_ppa_capable():
            add_ppa_repository("git-core/ppa")
        apt_get("git")
    if not exists("/usr/bin/git"):
        apt_get("git-core")

def _install_portable_git():
    from ..utils.windows import append_to_systemwide_path, get_homedir_windows_style
    from ..utils.unix import append_to_user_ssh_path_variable
    download_file_from_repository_on_remote_host("git/" + PORTABLE_GIT_ARCHIVE_NAME)
    untar(PORTABLE_GIT_ARCHIVE_NAME)
    run("chmod -R +wrx ~/git")
    append_to_systemwide_path("\\".join([get_homedir_windows_style(), "git", "bin"]))
    append_to_user_ssh_path_variable("~/git/bin")

def _install_darwin():
    from fabric.api import sudo
    sudo("rm /usr/bin/git")
    run("brew install git")

def _install_sunos():
    run("pkgutil -y -i git")

@task
def install():
    with user_context():
        from ..utils import determine_platform
        from ..utils.linux import determine_distribution
        if determine_platform() in ["linux", ]:
            dist, version, release = determine_distribution()
            if dist in ["ubuntu", ]:
                if release in ["natty", "oneiric", "raring", "quantal", ]:
                    # there are no packages for these releases on the ppa
                    _install_from_source()
                elif release in ["saucy", ]:
                    # apt-add-repository ppa:git-core/ppa doesn't work on these
                    _install_from_source()
                else:
                    _install_from_apt()
            elif dist in ["redhat", "centos", "suse"]:
                _install_from_source()
                run("chmod -R a+rX ~/git-{}".format(GIT_VERSION))
            else:
                abort("unsupported platform")
        elif determine_platform() in ["windows", ]:
            _install_portable_git()
        elif determine_platform() in ["darwin", ]:
            _install_darwin()
        elif determine_platform() in ["sunos", ]:
            _install_sunos()
        else:
            abort("unsupported platform")
