from os import path
from fabric.api import cd, puts, put, run, sudo, abort, settings, task
from fabric.contrib import files
from ..utils import user_context, get_and_put_file
from ..utils.linux import determine_distribution, apt_get, add_ppa_repository
from time import sleep

CONF = """
[[local|localrc]]
ADMIN_PASSWORD=stack
DATABASE_PASSWORD=$ADMIN_PASSWORD
RABBIT_PASSWORD=$ADMIN_PASSWORD
SERVICE_PASSWORD=$ADMIN_PASSWORD
SERVICE_TOKEN=$ADMIN_PASSWORD
LOGFILE=$DEST/logs/stack.sh.log
SCREEN_LOGDIR=$DEST/logs/screen
disable_service rabbit
enable_service zeromq

[[post-config|$CINDER_CONF]]
[DEFAULT]
#enabled_backends=
"""


def _git():
    from ..git import install as git_install, verify as git_verify
    if not git_verify():
        git_install()


def _multipath():
    from ..multipath import install as multipath_install, verify as multipath_verify
    if not multipath_verify():
        multipath_install()


def _mysql(distro):
    if distro in ('redhat', 'centos'):
        run("yum install -y mysql-server")
        run("service mysqld start")
        sleep(5)
        with settings(warn_only=True):
            run("mysqladmin -u root password stack")


def _redis(distro):
    if distro in ('redhat', 'centos'):
        # http://thoughts.z-dev.org/2013/05/27/install-redis-on-centos-6-4/
        run("wget http://dl.fedoraproject.org/pub/epel/6/x86_64/epel-release-6-8.noarch.rpm")
        run("wget http://rpms.famillecollet.com/enterprise/remi-release-6.rpm")
        run("rpm -Uvh remi-release-6*.rpm epel-release-6*.rpm || true")
        run("yum install redis -y")
        run("service redis start")
    elif distro in ('ubuntu'):
        add_ppa_repository("rwky/ppa")
        apt_get("redis-server")  # service is started automatically


def _sg3_utils(distro):
    if distro in ('redhat', 'centos'):
        for package in ('sg3_utils', 'sysfsutils', 'python-devel'):
            puts("installing package {}".format(package))
            run("yum install -y {}".format(package))
    elif distro in ('ubuntu'):
        for package in ('sg3-utils', 'sysfsutils', 'python-dev'):
            puts("installing package {}".format(package))
            apt_get(package)


def _python_dependencies(distro):
    if distro in ('redhat', 'centos'):
        # Install new distribute so we'll have easy_install on Python 2.6
        run("curl -q http://pypi01/media/dists/ez_setup.py | python2.6")
        # openstack needs redis Python package but does not install it by itself
        run("/usr/bin/easy_install pyzmq")
        run("/usr/bin/easy_install eventlet")
        run("/usr/bin/easy_install redis")


def _fixup_stackrc():
    run(r"sed -ie s'/https...github.com/git:\/\/infinigit.infinidat.com/' /opt/stack/devstack/stackrc")
    run(r"sed -ie s'/openstack-dev/openstack/' /opt/stack/devstack/stackrc")
    run(r"sed -ie s'/oslo./oslo-/' /opt/stack/devstack/stackrc")
    run(r"sed -ie s'/fujita/openstack/' /opt/stack/devstack/stackrc")
    run(r"sed -ie s'/tripleo/openstack/' /opt/stack/devstack/stackrc")
    run(r"sed -ie s'/kanaka/openstack/' /opt/stack/devstack/stackrc")
    run(r"sed -ie s'/osrg/openstack/' /opt/stack/devstack/stackrc")


def _write_local_conf_for_devstack():
    run("rm -f /opt/stack/devstack/localrc")
    with get_and_put_file("/opt/stack/devstack/local.conf") as local_conf:
        with open(local_conf, 'w') as fd:
            fd.write(CONF)
    run("chown stack /opt/stack/devstack/local.conf")


def _openstack(distro):
    run("/bin/rm -rf devstack")
    run("git clone git://infinigit.infinidat.com/openstack/devstack.git")
    run("cd devstack ; git checkout stable/havana")
    run("cd devstack ; sh ./tools/create-stack-user.sh")
    run("cd devstack ; mkdir -p /etc/sudoers.d; bash ./tools/create-stack-user.sh; cat /etc/sudoers.d/50_stack_sh >> /etc/sudoers")
    run("mv devstack /opt/stack/devstack ; chown -R stack /opt/stack/devstack ; chgrp -R stack /opt/stack/devstack")
    if distro in ('redhat', 'centos'):
        run("cp -f /etc/yum.conf{,.original}")
        run("echo gpgcheck=0 >> /etc/yum.conf")
    with settings(sudo_user='stack'):
        _fixup_stackrc()
        _write_local_conf_for_devstack()
        # sudo("cd /opt/stack/devstack ; chmod +x ./stack.sh; FORCE=yes ./stack.sh")
    if distro in ('redhat', 'centos'):
        run("cp -f /etc/yum.conf{.original,}")


@task
def start():
    with user_context():
        with settings(warn_only=True):
            run("service redis start")
        with settings(sudo_user='stack'):
            sudo("rm -f /opt/stack/devstack/localrc")
            sudo("rm -f /etc/cinder/cinder.conf")
            sudo("cd /opt/stack/devstack ; chmod +x ./stack.sh; FORCE=yes ./stack.sh")


@task
def stop():
    with user_context():
        with settings(sudo_user='stack'):
            sudo("cd /opt/stack/devstack ; chmod +x ./unstack.sh; ./unstack.sh")


@task
def install():
    distro, distro_version, _ = determine_distribution()
    if distro not in ('redhat', 'centos', 'ubuntu'):
        abort("distro {} not supported".format(distro))

    _git()
    _multipath()
    with user_context():
        _sg3_utils(distro)
        _python_dependencies(distro)
        _redis(distro)
        # _mysql(distro)
        _openstack(distro)
