from fabric.api import cd, put, run, abort, settings, task, puts, env
from ...download import download_file_from_repository_on_remote_host as download
from ..utils import user_context, get_tempfile, reconnect, is_solaris, untar, cmmi
from ..utils.unix import append_to_user_ssh_path_variable, append_to_bashrc
from infi.vendata.jslave.fabric.utils import get_and_put_file
from infi.vendata.jslave.fabric.utils import print_result

PYTHON_REPOSITORY = "ftp://repo.lab.il.infinidat.com/python"
PYTHON_VERSION = "v2.7.6.20"

@task
@print_result
def verify():
    from ..utils import determine_platform
    from ..utils.linux import determine_distribution
    from fabric.contrib.files import exists
    with user_context(), settings(warn_only=True):
        is_ubuntu = determine_platform() == "linux" and determine_distribution()[0] == "ubuntu"
        is_redhat_or_centos_7 = determine_platform() == "linux" and determine_distribution()[1].startswith('7')
        is_windows = determine_platform() == "windows"
        if is_ubuntu or is_redhat_or_centos_7:
            header_exists = any([exists("/root/python/include/python2.7/Python.h")])
            if not header_exists:
                return False
        if is_windows:
            if exists("/usr/bin/python"):
                return False
        expected_version = "2.7"
        return expected_version in run("python --version")

def _get_python_archive_name():
    from ..utils import determine_platform_string
    archive_name = "python-{}-{}.tar.gz".format(PYTHON_VERSION, determine_platform_string())
    return archive_name

def _install_from_download_repository():
    archive_name = _get_python_archive_name()
    download(archive_name, PYTHON_REPOSITORY)
    untar(archive_name)


def _build_isolated_python_from_source(python_executable="python"):
    run("rm -rf relocatable-python")
    run("git clone git://infinigit.infinidat.com/host/relocatable-python.git")
    with cd("relocatable-python"):
        run("git checkout origin/develop")
        run(python_executable + " bootstrap.py --setup-source=http://pypi01.infinidat.com/media/dists/ez_setup.py " + \
            "--index-url=http://pypi01.infinidat.com/simple --setuptools-version=7.0 " \
            "--download-base=http://pypi01.infinidat.com/media/dists")
        run("bin/buildout")
        run("bin/build")
    run("rm -rf python")
    run("mv relocatable-python/dist python")

def _build_python_tgz_and_then_build_isolated_python():
    run("rm -rf Python-2.7.7*")
    cmmi("sources/Python-2.7.7.tgz", PYTHON_REPOSITORY, "--prefix=/root/python_tgz")
    _build_isolated_python_from_source("/root/python_tgz/bin/python")

def _get_builtin_python():
    from fabric.contrib.files import exists
    for python in ["python2.3", "python2.4", "python2.6", "python2.7"]:
        if exists("/usr/bin/{}".format(python)):
            return python
    abort("no python found")

def _fix_yum_shbang():
    """yum calls python, and assumed it has the yum mudule. our python doesn't have it, so we need to update yum
    to use the builtin python"""
    from fabric.contrib.files import exists
    remote_path = "/usr/bin/yum"
    if not exists(remote_path):
        puts("there is no yum on host {}".format(env.host_string))
        return
    builtin_python = _get_builtin_python()
    with get_and_put_file(remote_path) as local_path:
        with open(local_path) as fd:
            content = fd.read().replace("#!/usr/bin/python\n", "#!/usr/bin/{}\n".format(builtin_python), 1)
        with open(local_path, 'w') as fd:
            fd.write(content)

def _fix_linux_environment_path():
    from re import sub, search, MULTILINE
    from ..utils import determine_architecture
    arch_name = "x64" if determine_architecture() == "x86_64" else "x86"
    defaults = '/usr/local/sbin:/usr/local/bin:/usr/sbin:/usr/bin:/sbin:/bin:/usr/games:/root/python'
    alias = "alias virtualenv='LD_LIBRARY_PATH=/root/python/lib{} virtualenv'"
    alias = alias.format("64" if arch_name == "x64" else '')
    append_to_bashrc(alias)
    append_to_user_ssh_path_variable('/root/python/bin', beginning=True)
    with get_and_put_file("/etc/environment") as local_environment:
        with open(local_environment) as fd:
            content = fd.read()
        path_pattern = '^PATH=\\"([\\w\\/\\:]+)\\"$'
        match = search(path_pattern, content, MULTILINE)
        elements = set(match.groups(1)[0].split(':')) if match else set()
        elements = elements.union(defaults.split(':'))
        elements = list(elements)
        elements.insert(0, "/root/python/bin")
        if match:
            new_content = sub(path_pattern, 'PATH="{}"'.format(':'.join(elements)), content, MULTILINE)
        else:
            new_content = 'PATH="{}"\n'.format(':'.join(elements))
        with open(local_environment, 'w') as fd:
            fd.write(new_content)
        with get_and_put_file("/etc/profile.local") as local_profile:
            with open(local_profile, 'w') as fd:
                fd.write('export PATH="/root/python/bin:$PATH"\n')

def _fix_solaris_environment_path():
    append_to_user_ssh_path_variable('/root/python/bin', beginning=True)

def _install_unix():
    try:
        _install_from_download_repository()
    except:
        try:
            _build_isolated_python_from_source()
        except:
            _build_python_tgz_and_then_build_isolated_python()

    if is_solaris():
        _fix_solaris_environment_path()
    else:
        _fix_yum_shbang()
        _fix_linux_environment_path()

    @reconnect
    def reconnect_to_update_path_variable():
        run("chmod -R a+rX ~/python")
    reconnect_to_update_path_variable()

def _install_windows():
    from ..utils import link
    from ..utils.windows import get_homedir_windows_style, append_to_systemwide_path
    _install_from_download_repository()
    run("rm /usr/bin/python || true")
    run("rm /usr/bin/python2.7 || true")
    run("chmod -R +wrx python")
    link("~/python/bin/python.exe", "~/python/bin/python2.7.exe")
    append_to_systemwide_path("\\".join([get_homedir_windows_style(), "python", "bin"]))
    append_to_systemwide_path("\\".join([get_homedir_windows_style(), "python", "scripts"]))
    append_to_user_ssh_path_variable("~/python/bin")
    append_to_user_ssh_path_variable("~/python/Scripts")

def _install_darwin():
    from fabric.api import sudo
    _install_from_download_repository()
    sudo("ln -f -s ~/python/bin/python2.7 /usr/bin/python2.7")
    sudo("ln -f -s /usr/bin/python2.7 /usr/bin/python")
    run("chmod a+rX ~")
    run("chmod -R a+rX ~/python")

@task
def install():
    with user_context():
        from ..utils import determine_platform
        if determine_platform() in ["linux", "sunos"]:
            _install_unix()
        elif determine_platform() in ["windows", ]:
            _install_windows()
        elif determine_platform() in ["darwin", ]:
            _install_darwin()
        else:
            abort("unknown operating system")

@task
def fix_environment():
    with user_context():
        _fix_linux_environment_path()
