from fabric.api import put, run, task, settings, abort, hide
from ...download import download_file_from_repository_on_remote_host as download
from ..utils import reconnect, get_tempfile, get_and_put_file
from ..utils.windows  import get_systemroot, is_windows_2003_or_xp, is_windows_core_edition
from infi.vendata.jslave.fabric.utils import print_result

def _install_vc_runtime():
    from ..utils import determine_architecture
    from ...download import SOFTWARE_REPOSITORY
    package_name = "vcredist_{}.exe".format("x64" if determine_architecture() == "x86_64" else "x86")
    download(package_name, '/'.join([SOFTWARE_REPOSITORY, 'vcredist', 'VisualStudio2008']))
    run("chmod +x {}".format(package_name))
    run("./{} /q /lang 1003".format(package_name))


def _install_dot_net_framework4__rgb9rast_component():
    from ..utils import determine_architecture
    from ...download import SOFTWARE_REPOSITORY
    package_name = "RGB9RAST_{}.msi".format("x64" if determine_architecture() == "x86_64" else "x86")
    download(package_name)
    run("chmod +x {}".format(package_name))
    cmd = r"msiexec /i {} /qn ALLUSERS=1 REBOOT=ReallySuppress /l*vx RGB9RAST.log"
    cmd = cmd.format(package_name)
    run(cmd)

def _install_dot_net_framework4():
    from ..utils import determine_architecture
    from ...download import SOFTWARE_REPOSITORY
    package_name = "dotNetFx40_Full_x86_x64{}.exe".format("_SC" if is_windows_core_edition() else "")
    download(package_name)
    run("chmod +x {}".format(package_name))
    cmd = r"./{} /passive /norestart /log C:DotNetFramework.log".format(package_name)
    run(cmd)

def _install_windows_imaging_component():
    from ..utils import determine_architecture
    from ...download import SOFTWARE_REPOSITORY
    package_name = "wic_{}_enu.exe".format("x64" if determine_architecture() == "x86_64" else "x86")
    download(package_name)
    run("chmod +x {}".format(package_name))
    run("./{} /quiet /norestart".format(package_name))

def _install_from_apt():
    from ..utils.linux import apt_get
    with settings(warn_only=True):
        run("apt-get update")
    apt_get("build-essential")
    apt_get("libxml2-dev")
    apt_get("libxslt-dev")
    apt_get("libevent-dev")
    apt_get("zlib1g-dev")
    apt_get("ftp")
    apt_get("strace")
    apt_get("gettext")

def _install_from_yum():
    run("yum install -y gcc-c++ gettext ftp make perl-ExtUtils-MakeMaker bzip2-devel zlib-devel patch rpm-build libxml2-devel libxslt-devel sg3_utils libevent-devel strace")

def _install_from_zypper():
    run("zypper --non-interactive --no-gpg-checks install --auto-agree-with-licenses --type pattern Basis-Devel")

def _install_devel_packages_from_zypper():
    run("zypper --non-interactive --no-gpg-checks install --auto-agree-with-licenses zlib-devel libbz2-devel libxml2-devel libxslt-devel libevent-devel strace gettext")

def _install_from_opencsw():
    admin_file_content = '\n'.join(['partial=nocheck',
                                    'runlevel=nocheck',
                                    'idepend=nocheck',
                                    'rdepend=nocheck',
                                    'setuid=nocheck',
                                    'action=nocheck',
                                    'instance=overwrite',
                                    'basedir=default'])
    admin_filename = '/tmp/admin_file'
    with get_and_put_file(admin_filename) as pkg_admin_file:
        open(pkg_admin_file, 'wb').write(admin_file_content)
    run("pkgadd -n -a {} -d http://get.opencsw.org/now CSWpkgutil".format(admin_filename))
    packages = ['gcc4g++', 'gmake', 'python27', 'libxml2_dev', 'libxslt_dev', 'libevent_dev', 'libbz2_dev', 'wget', 'gtar']
    [run("/opt/csw/bin/pkgutil -y -i {}".format(package)) for package in packages] # TODO sg3_utils

def _prepare_shell_files():
    for filepath in [".bash_profile", ".bash_history", ".bashrc", ".infinirc", "/etc/profile"]:
        run("touch {}".format(filepath))

def _fix_centos_4_base_repo(version):
    with get_and_put_file("/etc/yum.repos.d/CentOS-Base.repo") as base:
        with open(base) as fd:
            contents = fd.read()
        contents = contents.replace("#baseurl=", "baseurl=")
        contents = contents.replace("mirrorlist=", "#mirrorlist=")
        contents = contents.replace("gpgcheck=1", "gpgcheck=0")
        contents = contents.replace("mirror.centos.org/centos/$releasever",
                                    "vault.centos.org/" + version)
        with open(base, 'w') as fd:
            fd.write(contents)

def _install_libxml_from_source():
    from ..python import PYTHON_REPOSITORY
    from ..utils import cmmi
    cmmi("sources/libxml2-2.7.8.tar.gz", PYTHON_REPOSITORY)
    cmmi("sources/libxslt-1.1.26.tar.gz", PYTHON_REPOSITORY)

@task
def install():
    from ..utils import determine_platform, user_context
    from ..utils.linux import determine_distribution
    with user_context():
        run("rm -rf git*")
        if determine_platform() == "windows":
            _prepare_shell_files()
            if is_windows_2003_or_xp():
                _install_windows_imaging_component()
                _install_dot_net_framework4__rgb9rast_component()
            _install_dot_net_framework4()
        elif determine_platform() == "linux":
            dist, version , _ = determine_distribution()
            if dist in ["redhat", "centos"]:
                run("cp -f /etc/redhat-release /etc/centos-release || true")
                if version.startswith("4"):
                    _fix_centos_4_base_repo(version)
                _install_from_yum()
                if version.startswith("5"):
                    _install_libxml_from_source()
            elif dist in ["ubuntu", ]:
                _install_from_apt()
            elif dist in ["suse", ]:
                _install_from_zypper()
                if version.startswith("11"):
                    _install_devel_packages_from_zypper()
            else:
                abort("unsupported platform")
        elif determine_platform() == "darwin":
            run("brew install autoconf")
            run("brew install automake")
            run("brew install libtool")
        elif determine_platform() == "sunos":
            _install_from_opencsw()
        else:
            abort("unsupported platform")


def _verify_centos_release_file():
    from fabric.contrib.files import exists
    if not exists("/etc/redhat-release"):
        return True  # no redhat/centos, nothing to check here
    with get_tempfile("/etc/redhat-release") as redhat_release_file:
        with open(redhat_release_file) as fd:
            contents = fd.read()
        if "centos" in contents.lower() and not exists("/etc/centos-release"):
            return False
        return True


@task
@print_result
def verify():
    from ..utils import determine_platform, user_context
    from fabric.contrib.files import exists
    with user_context():
        if determine_platform() == "windows":
            return exists('/'.join([get_systemroot(), "Microsoft.NET", "Framework", "v4.0.30319"]))
        elif determine_platform() == "linux":
            return exists("/usr/bin/gcc") and _verify_centos_release_file() and \
                   exists("/usr/bin/ftp") and exists("/usr/bin/strace") and exists("/usr/bin/msgfmt")
        elif determine_platform() == "darwin":
            files = ["/usr/bin/gcc", "/usr/local/bin/autoconf", "/usr/local/bin/automake", "/usr/local/bin/glibtoolize"]
            return all([exists(file) for file in files])
        elif determine_platform() == "sunos":
            files = ["/opt/csw/bin/pkgutil", "/opt/csw/bin/python2.7", "/opt/csw/bin/gmake", "/opt/csw/bin/wget"]
            return all([exists(file) for file in files])
        else:
            abort("unsupported platform")
