import fabric
from fabric.api import task, run, settings, env, get, abort, hide, put, puts
from fabric.contrib.files import exists
from infi.pyutils.contexts import contextmanager
from infi.pyutils.decorators import wraps
from infi.pyutils.lazy import cached_function
from ...download import SOFTWARE_REPOSITORY
from logging import getLogger

class InvalidPassword(Exception):
    pass

DEFAULT_PASSWORDS_LIST = [
    ("Administrator", "Xsignnet1"), # Windows 2008 and north
    ("Administrator", "xsignnet1"), # Windows 2003
    ("root", "xsignnet1"), # unix
    ("jenkins", "jenkins"), # jenkins failback
    ]
logger = getLogger(__name__)

@contextmanager
def patch_prompt_for_password():
    from fabric.network import prompt_for_password
    def patch(*args, **kwargs):
        raise InvalidPassword
    fabric.network.prompt_for_password = patch
    try:
        yield None
    finally:
        fabric.network.prompt_for_password = prompt_for_password

@contextmanager
def user_context():
    for username, password in DEFAULT_PASSWORDS_LIST:
        with settings(user=username, password=password):
            if is_password_correct(env.host_string, username, password):
                yield username, password
                return
    abort("failed to determine username")

@task
def determine_user():
    with user_context() as user:
        return user[0]

@cached_function
def is_password_correct(host, username, password):
    with patch_prompt_for_password():
        try:
            _echo_hello_world()
            return True
        except InvalidPassword:
            return False

def _echo_hello_world():
    with settings(warn_only=True):
        run("echo hello world")

@cached_function
def _uname(host_string):
    return run("uname").stdout.strip("\n")

@cached_function
def _determine_platform_for_host(host_string):
    if exists("/cygdrive"):
        result = "Windows"
    else:
        result = _uname(host_string)
    puts("platform is {}".format(result))
    return result.lower()

@task
def determine_platform_string():
    from . import platform
    from infi.os_info import get_platform_string
    result = get_platform_string(platform)
    puts("platform_string is {!r}".format(result))
    return result

@task
def determine_platform():
    """:returns: "windows" or "linux" or "darwin" or "sunos" """
    with user_context():
        return _determine_platform_for_host(env.host_string)

@cached_function
def _determine_release_for_host(host_string):
    return run("uname -r").stdout.strip("\n")

@task
def determine_release():
    """:returns: os release number """
    with user_context():
        return _determine_release_for_host(env.host_string)

@cached_function
def _determine_architecture_for_host(host_string):
    if determine_platform() == "windows":
        result = "x86_64" if exists("/cygdrive/c/Windows/SysWOW64") else "i386"
    elif determine_platform() == "darwin":
        result = "x86_64"
    elif determine_platform() == "sunos":
        result = "sparc" if run("isainfo -n").stdout.startswith("sparc") else \
                 ("x86_64" if "64" in run("isainfo -b").stdout else "i386")
    elif _uname(host_string):
        result = "x86_64" if run("uname -i").stdout.startswith("x86_64") else "i386"
    else:
        abort("unsuppported operating system")
    puts("architecture is {}".format(result))
    return result

@task
def determine_architecture(xprefix=False):
    """:returns: "x86_64" or "i386" if xprefix is False, "x64" or "x86" if xprefix is True """
    with user_context():
        result = _determine_architecture_for_host(env.host_string)
        if not xprefix:
            return result
        return {"x86_64": "x64", "i386": "x86", "sparc": "sparc"}[result]

@task
def reboot():
    with user_context():
        if determine_platform() == "windows":
            run("shutdown -r -t 0 -f")
        elif determine_platform() == "linux":
            run("init 6")
        elif determine_platform() == "sunos":
            run("reboot")
        else:
            abort("unsuppported operating system")

@task
def link(src, dst):
    run("ln -f -s {} {}".format(src, dst))

@contextmanager
def get_tempfile(remote_path):
    from tempfile import mkstemp
    from os import close, remove
    fd, local_path = mkstemp()
    close(fd)
    if exists(remote_path):
        remove(local_path)
        get(remote_path, local_path)
    yield local_path
    remove(local_path)


@contextmanager
def get_and_put_file(remote_path):
    """this context manager:
    * gets remote_path as a tempfile when entering:
    * and putting it back when exiting"""
    with get_tempfile(remote_path) as local_path:
        yield local_path
        put(local_path, remote_path)

def reconnect(func):
    @wraps(func)
    def wrapper(*args, **kwargs):
        from fabric.network import disconnect_all
        disconnect_all()
        return func(*args, **kwargs)
    return wrapper

def reconnect_after(func):
    @wraps(func)
    def wrapper(*args, **kwargs):
        from fabric.network import disconnect_all
        res = func(*args, **kwargs)
        disconnect_all()
        return res
    return wrapper

def print_result(func):
    @wraps(func)
    def wrapper(*args, **kwargs):
        from fabric.api import env
        result = func(*args, **kwargs)
        puts("{}.{} result: {}".format(func.__module__.split('.')[-1], func.__name__, result))
        return result
    return wrapper

def check_ssh():
    from socket import create_connection
    name = env.host_string
    logger.info("Waiting for SSH service on {}".format(name))
    try:
        create_connection((name, 22))
        return True
    except Exception:
        return False

def wait_for_ssh(timeout_seconds=1800, sleep_seconds=60):
    from waiting import wait
    from fabric.network import disconnect_all
    wait(check_ssh, timeout_seconds=timeout_seconds, sleep_seconds=sleep_seconds)
    disconnect_all()

def is_windows():
    return determine_platform() in ["windows", ]

def is_solaris():
    return determine_platform() in ["sunos", ]

def untar(archive_name):
    tar_executable = 'gtar' if is_solaris() else 'tar'
    run("{} -xzf {}".format(tar_executable, archive_name))

def cmmi(archive_name, repository=SOFTWARE_REPOSITORY, configure_options=''):
    # cmmi = configure, make, make install
    import os
    from ...download import download_file_from_repository_on_remote_host as download
    from fabric.api import cd
    download(archive_name, repository)
    archive_basename = os.path.basename(archive_name)
    untar(archive_basename)
    archive_dirname = '.'.join(archive_basename.split(".")[:-2 if '.tar.gz' in archive_basename else -1])
    with cd(archive_dirname):
        run("./configure " + configure_options)
        run("make || make")
        run("make install")

from . import windows
from . import linux
