"""a platform-like module that implements system, architecture, processor, release, mac_ver, linux_distribution"""

from infi.pyutils.lazy import cached_function
from fabric.api import task, env, run, puts
from fabric.contrib.files import exists
from . import user_context

@cached_function
def _uname(host_string, arg=''):
    return run("uname %s" % arg).stdout.strip("\n")


@cached_function
def _call_python_platform_function_on_remote_host(host_string, func_name):
    cmdline = """python -c 'import platform; print repr(platform.%s())'""" % func_name
    return eval(run(cmdline).stdout)


@cached_function
def _determine_architecture_for_host(host_string, system):
    if system == "Windows":
        return ("64bit", ) if exists("/cygdrive/c/Windows/SysWOW64") else ("32bit", )
    elif system == "SunOS":
        return ("64bit", ) if run("isainfo -n").stdout.startswith("sparc") else \
                 ("64bit", ) if "64" in run("isainfo -b").stdout else ("32bit", )
    return _call_python_platform_function_on_remote_host(host_string, 'architecture')


@cached_function
def _linux_distribution(host_string):
    return  _call_python_platform_function_on_remote_host(host_string, 'linux_distribution')


@task
def system():
    """Returns the system/OS name, e.g. 'Linux', 'Windows', 'Darwin' or 'SunOS'. An empty string is returned if the value cannot be determined."""
    with user_context():
        result = _uname(env.host_string, '')
        if 'CYGWIN' in result:
            result = 'Windows'
        puts("system is {!r}".format(result))
        return result


@task
def architecture():
    """Returns a tuple (bits, linkage) which contain information about the bit architecture and the linkage format used for the executable. Both values are returned as strings"""
    _system = system()
    with user_context():
        result = _determine_architecture_for_host(env.host_string, _system)
        puts("architecture is {!r}".format(result))
        return result


@task
def processor():
    """Returns the (real) processor name, e.g. 'amdk6'."""
    with user_context():
        result = _uname(env.host_string, '-p')
        puts("processor is {!r}".format(result))
        return result


@task
def release():
    """Returns the system's release, e.g. '2.2.0' or 'NT' An empty string is returned if the value cannot be determined."""
    with user_context():
        result = _uname(env.host_string, '-r')
        puts("release is {!r}".format(result))
        return result


@task
def mac_ver():
    """Get Mac OS version information and return it as tuple (release, versioninfo, machine) with versioninfo being a tuple (version, dev_stage, non_release_version)."""
    with user_context():
        result = _call_python_platform_function_on_remote_host(host_string, 'mac_ver')
        puts("mac_ver is {!r}".format(result))
        return result


@task
def linux_distribution():
    with user_context():
        result = _linux_distribution(env.host_string)
        puts("linux_distribution is {!r}".format(result))
        return result
