from fabric.api import cd, put, run, abort, settings, task, puts
from fabric.contrib.files import exists
from ..utils import user_context, get_and_put_file, print_result, is_windows, is_solaris, is_solaris, get_tempfile, determine_release, cmmi
from ..utils.linux import determine_distribution, apt_get
from ..utils.windows import get_nt_version
import re

@task
@print_result
def verify():
    with user_context():
        return exists(".oh-my-zsh")


def fix_256color():
    # on solaris 10, 'export TERM=xterm-256color' won't work without this workaround
    src = "/opt/csw/share/terminfo/x/xterm-256color"
    dst = "/usr/share/lib/terminfo/x/xterm-256color"
    if not exists(dst) and exists(src):
        run("ln -s {} {}".format(src, dst))


def fix_dot_zsh():
    if is_windows():
        nt_versions = {"5.2": "2003", "6.0": "2008", "6.1": "2008 R2", "6.2": "2012", "6.3": "2012 R2"}
        os_name, os_version = "Windows", nt_versions[get_nt_version()]
    elif is_solaris():
        os_name, os_version = "SunOS", determine_release()
    else:
        os_name, os_version, _ = determine_distribution()
        os_name = os_name.capitalize()
    with get_and_put_file(".zshrc") as local_path:
        with open(local_path, 'rb') as fd:
            content = fd.read()
        # TODO: Check if this is still a problem: in windows bashrc looks wrong and we use bash_profile there anyway
        # source .bashrc
        source = "if [ -f ~/.infinirc ]; then\nsource ~/.infinirc\nfi\n"
        new_content = "\n".join([
            source,
            'DISABLE_AUTO_UPDATE="true"',
            'DISABLE_LS_COLORS="true"' if is_solaris() else '',
            content,
            'export PS1="%{{$fg[red]%}}%n@%m ({} {})%{{$reset_color%}} $PS1"'.format(os_name, os_version),
            'export RPROMPT="%F{11}%D %F{10}%D{%H:%M}%F{default}"',
            'export LC_ALL="en_US.UTF-8"',
            'export TERM=xterm-256color',
            ])
        new_content =  re.sub("\nplugins=\(.*?\)",
                          lambda matchobj: matchobj.group(0)[:-1] + " zsh-syntax-highlighting)",
                          new_content)

        with open(local_path, 'wb') as fd:
            fd.write(new_content)


def change_unix_shell():
    # with get_and_put_file("c:\\Cygwin\\Cygwin.bat") as local_path:
    #     with open(local_path, 'rb') as fd:
    #         content = fd.read()
    #     with open(local_path, 'wb') as fd:
    #         fd.write(content.replace("bash --login -i", "zsh -l -i"))
    if is_windows() or is_solaris():
        with get_and_put_file("/etc/passwd") as local_path:
            with open(local_path, 'rb') as fd:
                content = fd.read()
            with open(local_path, 'wb') as fd:
                fd.write(content.replace("/bin/bash", "/bin/zsh"))
    else:
        run("chsh -s /bin/zsh")


def fix_completion_warnings():
    if is_windows():
        run("chown -R Administrator /usr/share/zsh")
    if is_windows() or is_solaris():
        run("zsh -i -c 'compinit'")
        run("cp ./.zcompdump ./.zcompdump-$HOSTNAME-*")


def install_zsh_from_source():
    cmmi("zsh-5.0.6.tar.gz")
    run("rm -f /bin/zsh")
    run("link /usr/local/bin/zsh /bin/zsh")


def install_zsh():
    if is_windows():
        return
    if is_solaris():
        install_zsh_from_source()
        return
    dist, version, _ = determine_distribution()
    if dist in ["ubuntu", ]:
        apt_get("zsh")
    elif dist in ["redhat", "centos"]:
        if version in ["5.6", "4.8"]:
            run("yum install -y ncurses-devel")
            install_zsh_from_source()
        else:
            run("yum install -y zsh")
    elif dist in ["suse"]:
        install_zsh_from_source()
    else:
        abort("unsupported platform")


def install_oh_my_zsh():
    run("mkdir .oh-my-zsh")
    run("git clone git://github.com/robbyrussell/oh-my-zsh.git .oh-my-zsh")
    run("git clone git://github.com/zsh-users/zsh-syntax-highlighting.git "
        ".oh-my-zsh/custom/plugins/zsh-syntax-highlighting")
    run("cp ~/.oh-my-zsh/templates/zshrc.zsh-template ~/.zshrc")


@task
def install():
    with user_context():
        install_zsh()
        install_oh_my_zsh()
        fix_256color()
        fix_dot_zsh()
        change_unix_shell()
        fix_completion_warnings()
